<?php
/**
 * Date of Birth on Registration 0.2

 * Copyright 2010 Matthew Rogowski

 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at

 ** http://www.apache.org/licenses/LICENSE-2.0

 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
**/

if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook("member_register_start", "dbonreg_register");
$plugins->add_hook("member_do_register_start", "dobonreg");
$plugins->add_hook("usercp_do_profile_start", "dobonreg");

function dobonreg_info()
{
	return array(
		"name" => "Date of Birth on Registration",
		"description" => "Adds option to add your date of birth in the registration form.",
		"website" => "http://mattrogowski.co.uk",
		"author" => "MattRogowski",
		"authorsite" => "http://mattrogowski.co.uk",
		"version" => "0.2",
		"compatibility" => "16*",
		"guid" => "e212d0a10588d95c5b2c560d7f43e3ab"
	);
}

function dobonreg_activate()
{
	global $db;
	
	require_once MYBB_ROOT . "inc/adminfunctions_templates.php";
	
	dobonreg_deactivate();
	
	$settings_group = array(
		"name" => "dobonreg",
		"title" => "Date of Birth on Registration Settings",
		"description" => "Settings for the date of birth on registration plugin.",
		"disporder" => "28",
		"isdefault" => 0
	);
	$db->insert_query("settinggroups", $settings_group);
	$gid = $db->insert_id();
	
	$settings = array();
	$settings[] = array(
		"name" => "dobonreg_type",
		"title" => "Date of Birth Requirement",
		"description" => "What do you require users to enter??",
		"optionscode" => "select
required_full=Required - Full (DD/MM/YYYY)
required_partial=Required - Partial (DD/MM)
optional=Optional",
		"value" => "optional"
	);
	$settings[] = array(
		"name" => "dobonreg_agelimit",
		"title" => "Age Limit",
		"description" => "Do you want to limit registrations to a certain age?? If you want to limit users under 13, it is recommended to use the COPPA compliance feature of MyBB instead. Set to 0 for no age limit.",
		"optionscode" => "text",
		"value" => "0"
	);
	$i = 1;
	foreach($settings as $setting)
	{
		$insert = array(
			"name" => $db->escape_string($setting['name']),
			"title" => $db->escape_string($setting['title']),
			"description" => $db->escape_string($setting['description']),
			"optionscode" => $db->escape_string($setting['optionscode']),
			"value" => $db->escape_string($setting['value']),
			"disporder" => intval($i),
			"gid" => intval($gid),
		);
		$db->insert_query("settings", $insert);
		$i++;
	}
	
	rebuild_settings();
	
	$templates = array();
	$templates[] = array(
		"title" => "dobonreg",
		"template" => "<br />
<fieldset class=\"trow2\">
<legend><strong>{\$lang->date_of_birth}</strong></legend>
<table cellspacing=\"0\" cellpadding=\"{\$theme['tablespace']}\">
<tr>
<td><span class=\"smalltext\"><label for=\"referrer\">{\$dobonreg_desc}</label></span></td>
</tr>
<tr>
<td>
{\$dob_days} {\$dob_months} <label for=\"dob_year\" class=\"smalltext\">{\$lang->year}:</label> <input type=\"text\" name=\"dob_year\" id=\"dob_year\" class=\"textbox\" size=\"4\" maxlength=\"4\" value=\"{\$dob_year}\" />
</td>
</tr></table>
</fieldset>"
	);
	foreach($templates as $template)
	{
		$insert = array(
			"title" => $db->escape_string($template['title']),
			"template" => $db->escape_string($template['template']),
			"sid" => "-1",
			"version" => "1600",
			"dateline" => TIME_NOW
		);
		$db->insert_query("templates", $insert);
	}
	
	find_replace_templatesets("member_register", "#".preg_quote('{$requiredfields}')."#i", '{$requiredfields}{$dobonreg}');
}

function dobonreg_deactivate()
{
	global $db;
	
	require_once MYBB_ROOT . "inc/adminfunctions_templates.php";
	
	$db->delete_query("settinggroups", "name = 'dobonreg'");
	
	$settings = array(
		"dobonreg_type",
		"dobonreg_agelimit"
	);
	$settings = "'" . implode("','", $settings) . "'";
	$db->delete_query("settings", "name IN ({$settings})");
	
	rebuild_settings();
	
	$templates = array(
		"dobonreg"
	);
	$templates = "'" . implode("','", $templates) . "'";
	$db->delete_query("templates", "title IN ({$templates})");
	
	find_replace_templatesets("member_register", "#".preg_quote('{$dobonreg}')."#i", '', 0);
}

function dobonreg()
{
	global $plugins;
	
	$plugins->add_hook("datahandler_user_validate", "dobonreg_check");
}

function dbonreg_register()
{
	global $mybb, $lang, $templates, $dobonreg;
	
	$lang->load("dobonreg");
	
	switch($mybb->settings['dobonreg_type'])
	{
		case "required_full":
			$dobonreg_desc = $lang->dobonreg_full_required;
			break;
		case "required_partial":
			$dobonreg_desc = $lang->dobonreg_partial_required;
			break;
		default:
			$dobonreg_desc = $lang->dobonreg_optional;
	}
	
	$dob_days = "";
	$dob_days .= "<label for=\"dob_day\" class=\"smalltext\">{$lang->day}:</label> <select name=\"dob_day\" id=\"dob_day\">\n";
	$dob_days .= "<option value=\"\"></option>\n";
	for($i = 1; $i <= 31; $i++)
	{
		$dob_days .= "<option value=\"{$i}\">{$i}</option>\n";
	}
	$dob_days .= "</select>\n";
	
	$dob_months = "";
	$dob_months .= "<label for=\"dob_month\" class=\"smalltext\">{$lang->month}:</label> <select name=\"dob_month\" id=\"dob_month\">\n";
	$dob_months .= "<option value=\"\"></option>\n";
	for($i = 1; $i <= 12; $i++)
	{
		$month = "month_" . $i;
		$dob_months .= "<option value=\"{$i}\">{$lang->$month}</option>\n";
	}
	$dob_months .= "</select>\n";
	
	eval("\$dobonreg = \"".$templates->get('dobonreg')."\";");
}

function dobonreg_check() 
{
	global $mybb, $lang, $userhandler;
	
	$lang->load("dobonreg");
	
	$dob_day = intval($mybb->input['dob_day']);
	$dob_month = intval($mybb->input['dob_month']);
	$dob_year = intval($mybb->input['dob_year']);
	
	$birthday = array(
		"day" => $dob_day,
		"month" => $dob_month,
		"year" => $dob_year
	);
	
	if(!validate_birthday($birthday))
	{
		$userhandler->set_error("invalid_dob");
	}
	else
	{
		// Make the user's birthday field
		if($birthday['year'] != 0)
		{
			// If the year is specified, put together a d-m-y string
			$user['bday'] = $birthday['day']."-".$birthday['month']."-".$birthday['year'];
		}
		elseif($birthday['day'] && $birthday['month'])
		{
			// If only a day and month are specified, put together a d-m string
			$user['bday'] = $birthday['day']."-".$birthday['month']."-";
		}
		else
		{
			// No field is specified, so return an empty string for an unknown birthday
			$user['bday'] = '';
		}
		$userhandler->data['bday'] = $user['bday'];
	}
}

function validate_birthday($birthday)
{
	global $mybb, $lang, $userhandler;
	
	$lang->load("dobonreg");
	
	// a lot of this is copied from ./inc/datahandlers/user.php, but I wanted to change the error handling and check more stuff
	
	if($mybb->settings['dobonreg_type'] != "optional")
	{
		if(!$birthday['day'] || !$birthday['month'] || ($mybb->settings['dobonreg_type'] == "required_full" && !$birthday['year']))
		{
			if($mybb->settings['dobonreg_type'] == "required_full")
			{
				$userhandler->set_error("invalid_dob_need_full");
			}
			else
			{
				$userhandler->set_error("invalid_dob_empty");
			}
		}
	}
	
	if($mybb->settings['dobonreg_agelimit'] > 0)
	{
		$bday_time = @mktime(0, 0, 0, $birthday['month'], $birthday['day'], $birthday['year']);
		if($bday_time >= (mktime(0, 0, 0, my_date('n'), my_date('d'), my_date('Y') - intval($mybb->settings['dobonreg_agelimit']))))
		{
			error($lang->sprintf($lang->invalid_dob_age, $mybb->settings['dobonreg_agelimit']));
			return false;
		}
	}
	
	// error if a day and month exists, and the birthday day and range is not in range
	if($birthday['day'] != 0 || $birthday['month'] != 0)
	{
		if($birthday['day'] < 1 || $birthday['day'] > 31 || $birthday['month'] < 1 || $birthday['month'] > 12 || ($birthday['month'] == 2 && $birthday['day'] > 29))
		{
			$userhandler->set_error("invalid_dob");
			return false;
		}
	}

	// Check if the day actually exists.
	$months = get_bdays($birthday['year']);
	if($birthday['day'] > $months[$birthday['month']-1])
	{
		$userhandler->set_error("invalid_dob");
		return false;
	}

	// Error if a year exists and the year is out of range
	if($birthday['year'] != 0 && ($birthday['year'] < (date("Y")-100)) || $birthday['year'] > date("Y"))
	{
		$userhandler->set_error("invalid_dob");
		return false;
	}
	elseif($birthday['year'] == date("Y"))
	{
		// Error if birth date is in future
		if($birthday['month'] > date("m") || ($birthday['month'] == date("m") && $bithday['day'] > date("d")))
		{
			$userhandler->set_error("invalid_dob");
			return false;
		}
	}
	
	return true;
}
?>